@@ -49,7 +49,7 @@ static void acpi_sensors_interface_add_sensor(SensorsApplet *sensors_applet, con
 				  ACPI,
 				  TRUE,
 				  TEMP_SENSOR,
-				  CPU_TEMP_ICON);
+				  CPU_ICON);
 	g_free(id);
 }	
 
@@ -145,5 +145,5 @@ gdouble acpi_sensors_interface_get_sensor_value(const gchar *path,
 		sensor_value = (sensor_value / 10.0) - 273.0;
 	}
 
-	return sensor_value;
+	return (gdouble)sensor_value;
 }
