@@ -20,10 +20,21 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 
 #include "hddtemp-sensors-interface.h"
 #include "sensors-applet.h"
@@ -129,7 +140,7 @@ static void hddtemp_sensors_interface_get_sensors(SensorsApplet *sensors_applet)
 	}
 
 	if (hddtemp_output[0] != '|') {
-		g_printf("Error in format of string returned from hddtemp daemon: char at [0] should be \"|\", instead whole output is: \"%s\"", hddtemp_output);
+		g_debug("Error in format of string returned from hddtemp daemon: char at [0] should be \"|\", instead whole output is: \"%s\"", hddtemp_output);
 		return;
 	}
 
@@ -159,7 +170,7 @@ static void hddtemp_sensors_interface_get_sensors(SensorsApplet *sensors_applet)
 						  HDDTEMP,
 						  FALSE,
 						  TEMP_SENSOR,
-						  HDD_TEMP_ICON);
+						  HDD_ICON);
 		}
 		pv += 5; 
 	}
@@ -198,7 +209,7 @@ gdouble hddtemp_sensors_interface_get_sensor_value(const gchar *path,
 	}
 
 	if (hddtemp_output[0] != '|') {
-		g_printf("Error in format of string returned from hddtemp daemon: char at [0] should be \"|\", instead whole output is: \"%s\"", hddtemp_output);
+		g_debug("Error in format of string returned from hddtemp daemon: char at [0] should be \"|\", instead whole output is: \"%s\"", hddtemp_output);
 		return sensor_value;
 	}
 
@@ -231,5 +242,5 @@ gdouble hddtemp_sensors_interface_get_sensor_value(const gchar *path,
 	}
 	g_strfreev(output_vector);
 	
-	return sensor_value;
+	return (gdouble)sensor_value;
 }
