@@ -48,7 +48,7 @@ static void i2c_proc_sensors_interface_add_sensor(SensorsApplet *sensors_applet,
 		sensor_type = TEMP_SENSOR;
 		label = g_strdup(_("CPU"));
 		enable = TRUE;
-		icon_filename = CPU_TEMP_ICON;
+		icon_filename = CPU_ICON;
 	} else {
 		label = g_strdup(filename);
 		
@@ -66,7 +66,6 @@ static void i2c_proc_sensors_interface_add_sensor(SensorsApplet *sensors_applet,
 			break;
 		case 't':     /* temps are "temp?" */
 			sensor_type = TEMP_SENSOR;
-			icon_filename = GENERIC_TEMP_ICON;
 			break;
 		case 'v':     /* vids are just vid */
 			sensor_type = VOLTAGE_SENSOR;
