@@ -49,7 +49,7 @@ static void i2c_sys_sensors_interface_add_sensor(SensorsApplet *sensors_applet,
 		label = g_strdup(_("CPU"));
 		sensor_type = TEMP_SENSOR;
 		enable = TRUE;
-		icon_filename = CPU_TEMP_ICON;
+		icon_filename = CPU_ICON;
 	} else {
 		switch(filename[0]) {
 		case 'c':
@@ -76,7 +76,6 @@ static void i2c_sys_sensors_interface_add_sensor(SensorsApplet *sensors_applet,
 			id = g_strndup(filename, 5);
 			label = g_strndup(filename, 5);
 			sensor_type = TEMP_SENSOR;
-			icon_filename = GENERIC_TEMP_ICON;
 			break;
 		case 'i':     /* voltages are "in?_input" */
 			id = g_strndup(filename, 3);
