@@ -46,7 +46,7 @@ static void i8k_sensors_interface_setup_manually(SensorsApplet *sensors_applet)
 					  I8K,
 					  TRUE,
 					  TEMP_SENSOR,
-					  CPU_TEMP_ICON);
+					  CPU_ICON);
 		sensors_applet_add_sensor(sensors_applet,
 					  I8K_DEVICE_PATH,
 					  "fan1",
@@ -109,7 +109,7 @@ gdouble i8k_sensors_interface_get_sensor_value(const gchar *path,
 
 	/* count spaces but stop if have counted 100 characters and
 	   still not found a space (to avoid infinite loop if file
-	   format error)*/
+	   format error) */
 	while (file_length < 100 && space_count < 3) {
 		if (fgetc(fp) == ' ') {
 			space_count++;
@@ -145,6 +145,6 @@ gdouble i8k_sensors_interface_get_sensor_value(const gchar *path,
 		g_assert_not_reached();			
 	} // end switch (sensor_type)
 
-	return (gfloat)sensor_value;
+	return (gdouble)sensor_value;
 
 }
