@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
+ * Copyright (C) 2005 Alex Murray <pragmatine@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 
 /* function definitions */
 void i8k_sensors_interface_init(SensorsApplet *sensors_applet);
-gfloat i8k_sensors_interface_get_sensor_value(const gchar *path, 
+gdouble i8k_sensors_interface_get_sensor_value(const gchar *path, 
 					      const gchar *id, 
 					      SensorType type,
 					      GError **error);
