@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 Alex Murray <murray.alex@gmail.com>
+ * Copyright (C) 2005 Alex Murray <pragmatine@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ static gboolean sensors_applet_fill(PanelApplet *applet, const gchar *iid, gpoin
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:SensorsApplet_Factory", 
 			     PANEL_TYPE_APPLET, 
-			     PACKAGE_NAME, 
+			     PACKAGE, 
 			     PACKAGE_VERSION, 
 			     sensors_applet_fill, 
 			     NULL);
