@@ -44,7 +44,7 @@ static void omnibook_sensors_interface_setup_manually(SensorsApplet *sensors_app
 					  OMNIBOOK,
 					  TRUE,
 					  TEMP_SENSOR,
-					  CPU_TEMP_ICON);
+					  CPU_ICON);
 	}
 }	
 
