@@ -55,12 +55,12 @@ static void pmu_sys_sensors_interface_add_sensor(SensorsApplet *sensors_applet,
 		label = g_strdup(_("CPU"));
 		sensor_type = TEMP_SENSOR;
 		enable = TRUE;
-		icon_filename = CPU_TEMP_ICON;
+		icon_filename = CPU_ICON;
 	} else if (g_ascii_strcasecmp(filename, SENSOR2 TEMPERATURE) == 0 || g_ascii_strcasecmp(filename, GPU TEMPERATURE) == 0) {
 		label = g_strdup(_("GPU"));
 		sensor_type = TEMP_SENSOR;
 		enable = TRUE;
-		icon_filename = GPU_TEMP_ICON;
+		icon_filename = GPU_ICON;
 	} else if (g_ascii_strcasecmp(filename, SENSOR1 FAN_SPEED) == 0|| g_ascii_strcasecmp(filename, CPU FAN_SPEED) == 0) { 
 		label = g_strdup(_("FAN"));
 		sensor_type = FAN_SENSOR;
