@@ -37,44 +37,43 @@ static void prefs_dialog_close(SensorsApplet *sensors_applet) {
 		sensors_applet->prefs_dialog = NULL;
 	}
 	if (sensors_applet->timeout_id == 0) {
-		sensors_applet->timeout_id = g_timeout_add(panel_applet_gconf_get_int(sensors_applet->applet, TIMEOUT, NULL), (GSourceFunc)sensors_applet_update_display, sensors_applet);
+		sensors_applet->timeout_id = g_timeout_add(panel_applet_gconf_get_int(sensors_applet->applet, TIMEOUT, NULL), (GSourceFunc)sensors_applet_update_active_sensors, sensors_applet);
 	}
 
 }
 
-static void prefs_dialog_farenheit_toggled(GtkToggleButton *button, SensorsApplet *sensors_applet) {
+static void prefs_dialog_Fahrenheit_toggled(GtkToggleButton *button, SensorsApplet *sensors_applet) {
 	gboolean value;
 	g_object_get(button, "active", &value, NULL);
 	panel_applet_gconf_set_bool(sensors_applet->applet, FARENHEIT, value, NULL);
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_active_sensors(sensors_applet);
 }
 		
 static void prefs_dialog_labels_inline_toggled(GtkToggleButton *button, SensorsApplet *sensors_applet) {
 	gboolean value;
 	g_object_get(button, "active", &value, NULL);
 	panel_applet_gconf_set_bool(sensors_applet->applet, LABELS_INLINE, value, NULL);
-	sensors_applet_update_display(sensors_applet, TRUE);
+	sensors_applet_display_layout_changed(sensors_applet);
 }
 
 static void prefs_dialog_show_units_toggled(GtkToggleButton *button, SensorsApplet *sensors_applet) {
 	gboolean value;
 	g_object_get(button, "active", &value, NULL);
 	panel_applet_gconf_set_bool(sensors_applet->applet, SHOW_UNITS, value, NULL);
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_active_sensors(sensors_applet);
 }
 
 static void prefs_dialog_num_sensors_changed(GtkSpinButton *button, SensorsApplet *sensors_applet) {
 	gint value;
 	value = gtk_spin_button_get_value_as_int(button);
 	panel_applet_gconf_set_int(sensors_applet->applet, NUM_SENSORS, value, NULL);
-	sensors_applet_update_display(sensors_applet, TRUE);
+	sensors_applet_display_layout_changed(sensors_applet);
 }
 
 static void prefs_dialog_timeout_changed(GtkSpinButton *button, SensorsApplet *sensors_applet) {
 	gint value;
 	value = (gint)(gtk_spin_button_get_value(button) * 1000);
 	panel_applet_gconf_set_int(sensors_applet->applet, TIMEOUT, value, NULL);
-
 }
 
 static void prefs_dialog_display_mode_changed(GtkComboBox *display_mode_combo_box,
@@ -84,7 +83,7 @@ static void prefs_dialog_display_mode_changed(GtkComboBox *display_mode_combo_bo
 				   gtk_combo_box_get_active(display_mode_combo_box),
 				   NULL);
 	
-	sensors_applet_update_display(sensors_applet, TRUE);
+	sensors_applet_display_layout_changed(sensors_applet);
 }
 
 static void prefs_dialog_font_size_changed(GtkComboBox *font_size_combo_box,
@@ -94,33 +93,30 @@ static void prefs_dialog_font_size_changed(GtkComboBox *font_size_combo_box,
 				   gtk_combo_box_get_active(font_size_combo_box),
 				   NULL);
 	
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_active_sensors(sensors_applet);
 }
 
 /* callbacks for the tree of sensors */
 static void prefs_dialog_sensor_toggled(GtkCellRenderer *renderer, gchar *path_str, SensorsApplet *sensors_applet) {
 	GtkTreeIter iter;
-	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
+	GtkTreePath *path;
+
 	gboolean old_value;
-	gint alarm_timeout_index;
+
+	path = gtk_tree_path_new_from_string(path_str);
 	
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(sensors_applet->sensors), &iter, path);
 	gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors), 
 			   &iter, 
 			   ENABLE_COLUMN, &old_value, 
-			   ALARM_TIMEOUT_INDEX_COLUMN, &alarm_timeout_index,
 			   -1);
 
-	if (old_value == FALSE && alarm_timeout_index != -1) {
-		/* alarm is on but about to turn off sensor so need to
-		   kill alarm also */
-		if (!g_source_remove(alarm_timeout_index)) {
-			g_printf("Error removing alarm source\n");
-		}
-		gtk_tree_store_set(sensors_applet->sensors,
-				   &iter,
-				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
-				   -1);
+	if (old_value) {
+		sensors_applet_sensor_disabled(sensors_applet,
+					       path);
+	} else {
+		sensors_applet_sensor_enabled(sensors_applet,
+					      path);
 	}
 	
 	gtk_tree_store_set(sensors_applet->sensors, &iter, 
@@ -128,10 +124,6 @@ static void prefs_dialog_sensor_toggled(GtkCellRenderer *renderer, gchar *path_s
 			   -1);
 
 	gtk_tree_path_free(path);
-
-	/* need to update entire display if a sensor has been added /
-	   removed */
-	sensors_applet_update_display(sensors_applet, TRUE);
 }
 
 static void prefs_dialog_sensor_name_changed(GtkCellRenderer *renderer, gchar *path_str, gchar *new_text, SensorsApplet *sensors_applet) {
@@ -142,9 +134,8 @@ static void prefs_dialog_sensor_name_changed(GtkCellRenderer *renderer, gchar *p
 
 	gtk_tree_store_set(sensors_applet->sensors, &iter, LABEL_COLUMN, new_text, -1);
 		
+	sensors_applet_update_sensor(sensors_applet, path);
 	gtk_tree_path_free(path);
-
-	sensors_applet_update_display(sensors_applet, FALSE);
 }
 
 static void prefs_dialog_sensor_config_button_clicked(GtkButton *button, SensorsApplet *sensors_applet) {
@@ -177,7 +168,7 @@ void prefs_dialog_open(SensorsApplet *sensors_applet) {
 	GtkHSeparator *globals_separator;
 	GtkComboBox *font_size_combo_box, *display_mode_combo_box;
 	GtkLabel *num_sensors_label, *timeout_label, *font_size_label, *display_mode_label;
-	GtkCheckButton *farenheit_checkbutton, *labels_inline_checkbutton, *show_units_checkbutton;
+	GtkCheckButton *Fahrenheit_checkbutton, *labels_inline_checkbutton, *show_units_checkbutton;
 	GtkAdjustment *num_sensors_adjust, *timeout_adjust; 
 
 	/* widgets for sensors tree */
@@ -261,13 +252,13 @@ void prefs_dialog_open(SensorsApplet *sensors_applet) {
 	g_signal_connect(show_units_checkbutton, "toggled", G_CALLBACK(prefs_dialog_show_units_toggled), sensors_applet);
 	
 	
-	farenheit_checkbutton = g_object_new(GTK_TYPE_CHECK_BUTTON,
+	Fahrenheit_checkbutton = g_object_new(GTK_TYPE_CHECK_BUTTON,
 					     "use-underline", TRUE,
-					     "label", _("Use _farenheit for temperature sensors"),
+					     "label", _("Use _Fahrenheit for temperature sensors"),
 					     "active", panel_applet_gconf_get_bool(sensors_applet->applet, FARENHEIT, NULL),
 					     NULL);
 	
-	g_signal_connect(farenheit_checkbutton, "toggled", G_CALLBACK(prefs_dialog_farenheit_toggled), sensors_applet);
+	g_signal_connect(Fahrenheit_checkbutton, "toggled", G_CALLBACK(prefs_dialog_Fahrenheit_toggled), sensors_applet);
 	
 
 	num_sensors_adjust = g_object_new(GTK_TYPE_ADJUSTMENT,
@@ -500,7 +491,7 @@ void prefs_dialog_open(SensorsApplet *sensors_applet) {
 			 0);
 
 	gtk_table_attach(globals_table,
-			 GTK_WIDGET(farenheit_checkbutton),
+			 GTK_WIDGET(Fahrenheit_checkbutton),
 			 0, 2,
 			 6, 7,
 			 GTK_FILL,
