@@ -24,8 +24,12 @@
 #include "sensor-config-dialog.h"
 #include "sensors-applet.h"
 
+#define SPINBUTTON_WIDTH_CHARS 7
+#define VALUE_DECIMAL_PLACES 3
+
 static void sensor_config_dialog_icon_changed(GnomeIconEntry *icon_entry, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	GdkPixbuf *new_icon;
 	gchar *new_filename;
@@ -34,6 +38,8 @@ static void sensor_config_dialog_icon_changed(GnomeIconEntry *icon_entry, Sensor
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
 
 	new_filename = gnome_icon_entry_get_filename(icon_entry);
 	new_icon = gdk_pixbuf_new_from_file_at_size(new_filename, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE, &error);
@@ -61,12 +67,14 @@ static void sensor_config_dialog_icon_changed(GnomeIconEntry *icon_entry, Sensor
 	/* tree now holds ref */
 	g_object_unref(new_icon);
 	g_free(new_filename);
-
-	sensors_applet_update_display(sensors_applet, FALSE);
+	
+	sensors_applet_icon_changed(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 static void sensor_config_dialog_multiplier_changed(GtkSpinButton *spinbutton, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	gdouble value;
 
@@ -75,16 +83,21 @@ static void sensor_config_dialog_multiplier_changed(GtkSpinButton *spinbutton, S
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
+
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
 	gtk_tree_store_set(sensors_applet->sensors,
 			    &iter,
 			    MULTIPLIER_COLUMN, value,
 			   -1);
 
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 static void sensor_config_dialog_offset_changed(GtkSpinButton *spinbutton, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	gdouble value;
 
@@ -93,16 +106,20 @@ static void sensor_config_dialog_offset_changed(GtkSpinButton *spinbutton, Senso
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
 	gtk_tree_store_set(sensors_applet->sensors,
 			    &iter,
 			    OFFSET_COLUMN, value,
 			   -1);
 
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 static void sensor_config_dialog_alarm_type_changed(GtkComboBox *alarm_type_combo_box, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	guint value;
 
@@ -111,16 +128,20 @@ static void sensor_config_dialog_alarm_type_changed(GtkComboBox *alarm_type_comb
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
 	gtk_tree_store_set(sensors_applet->sensors,
 			    &iter,
 			    ALARM_TYPE_COLUMN, value,
 			   -1);
 
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 
 }
 static void sensor_config_dialog_alarm_value_changed(GtkSpinButton *spinbutton, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	gdouble value;
 
@@ -129,17 +150,21 @@ static void sensor_config_dialog_alarm_value_changed(GtkSpinButton *spinbutton,
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
 
 	gtk_tree_store_set(sensors_applet->sensors,
 			    &iter,
 			    ALARM_VALUE_COLUMN, value,
 			   -1);
 
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 static void sensor_config_dialog_alarm_toggled(GtkToggleButton *button, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 
 	gboolean value;
@@ -149,16 +174,21 @@ static void sensor_config_dialog_alarm_toggled(GtkToggleButton *button, SensorsA
 					&model,
 					&iter);
 	
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
+
 	gtk_tree_store_set(sensors_applet->sensors,
 			    &iter,
 			    ALARM_ENABLE_COLUMN, value,
 			   -1);
 
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 static void sensor_config_dialog_alarm_timeout_changed(GtkSpinButton *spinbutton, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	gint value;
 
@@ -167,17 +197,22 @@ static void sensor_config_dialog_alarm_timeout_changed(GtkSpinButton *spinbutton
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
-	sensors_applet_alarm_off(sensors_applet, &iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
+
+	sensors_applet_alarm_off(sensors_applet, path);
 	gtk_tree_store_set(sensors_applet->sensors,
 			    &iter,
 			    ALARM_TIMEOUT_COLUMN, value,
 			   -1);
 
-	sensors_applet_update_display(sensors_applet, FALSE);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 static void sensor_config_dialog_alarm_command_edited(GtkEntry *command_entry, SensorsApplet *sensors_applet) {
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	
 	gchar *value;
@@ -186,13 +221,18 @@ static void sensor_config_dialog_alarm_command_edited(GtkEntry *command_entry, S
 	gtk_tree_selection_get_selected(sensors_applet->selection,
 					&model,
 					&iter);
-	sensors_applet_alarm_off(sensors_applet, &iter);
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors),
+				       &iter);
+
+	sensors_applet_alarm_off(sensors_applet, path);
 
 	gtk_tree_store_set(sensors_applet->sensors,
 			   &iter,
 			   ALARM_COMMAND_COLUMN, value,
 			   -1);
 	g_free(value);
+	sensors_applet_update_sensor(sensors_applet, path);
+	gtk_tree_path_free(path);
 }
 
 void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_applet) {
@@ -265,9 +305,9 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 	/* do multiplier and offset widgets */
 	multiplier_adjust = g_object_new(GTK_TYPE_ADJUSTMENT,
 			      "value", 1.0,
-			      "lower", 0.1,
-			      "upper", 100.0,
-			      "step-increment", 0.01,
+			      "lower", 0.001,
+			      "upper", 1000.0,
+			      "step-increment", 0.1,
 			      "page-increment", 1.0,
 			      "page-size", 1.0,
 			      NULL);
@@ -275,9 +315,9 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 	
 	multiplier_spinbutton = g_object_new(GTK_TYPE_SPIN_BUTTON,
 					     "adjustment", multiplier_adjust,
-					     "digits", 2,
+					     "digits", VALUE_DECIMAL_PLACES,
 					     "value", multiplier,
-					     "width-chars", 6,
+					     "width-chars", SPINBUTTON_WIDTH_CHARS,
 					     NULL);
 
 	multiplier_spinbutton_aligner = g_object_new(GTK_TYPE_ALIGNMENT,
@@ -299,8 +339,8 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 
 	offset_adjust = g_object_new(GTK_TYPE_ADJUSTMENT,
 				     "value", 0.0,
-				     "lower", -1000.00,
-				     "upper", 1000.0,
+				     "lower", -1000.000,
+				     "upper", 1000.000,
 				     "step-increment", 0.01,
 				     "page-increment", 1.0,
 				     "page-size", 1.0,
@@ -308,9 +348,9 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 
 	offset_spinbutton = g_object_new(GTK_TYPE_SPIN_BUTTON,
 					 "adjustment", offset_adjust,
-					 "digits", 2,
+					 "digits", VALUE_DECIMAL_PLACES,
 					 "value", (gdouble)offset,
-					 "width-chars", 6,
+					 "width-chars", SPINBUTTON_WIDTH_CHARS,
 					 NULL);
 	
 	offset_spinbutton_aligner = g_object_new(GTK_TYPE_ALIGNMENT,
@@ -358,8 +398,8 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 
 	alarm_value_adjust = g_object_new(GTK_TYPE_ADJUSTMENT,
 			      "value", 0.0,
-			      "lower", 0.0,
-			      "upper", 10000.0,
+			      "lower", -100000.0,
+			      "upper", 100000.0,
 			      "step-increment", 1.0,
 			      "page-increment", 10.0,
 			      "page-size", 100.0,
@@ -368,9 +408,9 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 	
 	alarm_value_spinbutton = g_object_new(GTK_TYPE_SPIN_BUTTON,
 					      "adjustment", alarm_value_adjust,
-					      "digits", 1,
+					      "digits", VALUE_DECIMAL_PLACES,
 					      "value", alarm_value,
-					      "width-chars", 6,
+					      "width-chars", SPINBUTTON_WIDTH_CHARS,
 					      NULL);
 	
 	alarm_value_spinbutton_aligner = g_object_new(GTK_TYPE_ALIGNMENT,
@@ -402,7 +442,7 @@ void sensor_config_dialog_create(GtkButton *button, SensorsApplet *sensors_apple
 						"adjustment", alarm_timeout_adjust,
 						"digits", 0,
 						"value", (gdouble)alarm_timeout,
-						"width-chars", 6,
+						"width-chars", SPINBUTTON_WIDTH_CHARS,
 						NULL);
 	alarm_timeout_spinbutton_aligner = g_object_new(GTK_TYPE_ALIGNMENT,
 							"child", alarm_timeout_spinbutton,
