@@ -49,9 +49,9 @@ static void sensors_applet_gconf_error_occurred(void) {
 			    "yalign", 1.0,
 			    NULL);
 	
-	markup = g_markup_printf_escaped(_("<b>An error occurred trying to reload the sensors.</b>\n\
-Did you forget to load the correct module?  Once the problem\n\
-is resolved you will need to reload %s."), PACKAGE_NAME);
+	markup = g_markup_printf_escaped(_("<b>An error occurred trying to reload the sensors.</b>\n"
+					   "Did you forget to load the correct module?  Once the problem\n"
+					   "is resolved you will need to reload %s."), PACKAGE_NAME);
 
 	label = g_object_new(GTK_TYPE_LABEL,
 			     "wrap", TRUE,
@@ -181,6 +181,7 @@ gboolean sensors_applet_gconf_setup_sensors(SensorsApplet *sensors_applet) {
 	     current_path != NULL && current_id != NULL && current_label != NULL && current_enable != NULL && current_interface != NULL && current_alarm_value != NULL && current_alarm_type != NULL && current_alarm_enable != NULL && current_alarm_command != NULL && current_alarm_timeout != NULL && current_sensor_type != NULL && current_multiplier != NULL && current_offset != NULL && current_icon != NULL;
 	     current_path = g_slist_next(current_path), current_id = g_slist_next(current_id), current_label = g_slist_next(current_label), current_enable = g_slist_next(current_enable), current_interface = g_slist_next(current_interface), current_alarm_value = g_slist_next(current_alarm_value), current_alarm_type = g_slist_next(current_alarm_type), current_alarm_enable = g_slist_next(current_alarm_enable), current_alarm_command = g_slist_next(current_alarm_command), current_alarm_timeout = g_slist_next(current_alarm_timeout), current_sensor_type = g_slist_next(current_sensor_type), current_multiplier = g_slist_next(current_multiplier), current_offset = g_slist_next(current_offset), current_icon = g_slist_next(current_icon)) {
 
+		g_debug("trying to add sensor from gconf data: %s\n", (gchar *)(current_id->data));
 		sensors_applet_add_sensor_full_details(sensors_applet,
 						       (gchar *)(current_path->data), 
 						       (gchar *)(current_id->data), 
