@@ -27,14 +27,23 @@
 #include <gnome.h>
 #include <glib/gprintf.h>
 #include "sensors-applet.h"
+#include "active-sensor.h"
 #include "sensors-applet-gconf.h"
 #include "acpi-sensors-interface.h"
+
+/* use libsensors if available, otherwise manually find sensors */
+#ifdef HAVE_LIBSENSORS
+#include "libsensors-sensors-interface.h"
+#else
 #include "i2c-sys-sensors-interface.h"
 #include "i2c-proc-sensors-interface.h"
+#endif
+
 #include "i8k-sensors-interface.h"
 #include "ibm-acpi-sensors-interface.h"
 #include "omnibook-sensors-interface.h"
 #include "pmu-sys-sensors-interface.h"
+#include "smu-sys-sensors-interface.h"
 #include "hddtemp-sensors-interface.h"
 #include "prefs-dialog.h"
 #include "about-dialog.h"
@@ -74,7 +83,7 @@ static void help_cb (BonoboUIComponent *uic,
 			      &error);
 
 	   if (error) {
-			 g_printf("Could not open help document: %s\n ",error->message);
+			 g_debug("Could not open help document: %s\n ",error->message);
 			 g_error_free (error);
 	   }
 }
@@ -184,6 +193,9 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 	gchar *sensor_id;
 	GdkPixbuf *icon;
 	GError *error = NULL;
+	GtkTreePath *tree_path;
+	GtkTreeRowReference *row;
+	ActiveSensor *active_sensor;
 
 	g_assert(sensors_applet != NULL);
 
@@ -207,15 +219,13 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 							     G_TYPE_BOOLEAN, /* alarm enable */
 							     G_TYPE_STRING, /* alarm command */ 
 							     G_TYPE_UINT, /* alarm timeout */
-							     G_TYPE_INT, /* alarm
-									  * timeout
-									  * index */
 							     G_TYPE_DOUBLE, /* multiplier */
 							     G_TYPE_DOUBLE, /* offset */
 							     G_TYPE_STRING, /* icon filename */
 							     GDK_TYPE_PIXBUF); /* icon pixbuf */
+		      
 		 
-		
+		g_debug("Sensor tree created.\n");
 
 		/* we know tree is actually empty since we just created it */
 		not_empty_tree = FALSE;
@@ -242,6 +252,7 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 				if (g_ascii_strcasecmp(sensor_id, id) == 0) {
 					/* sensor already exists so
 					 * dont add a second time */
+					g_debug("sensor already exists in tree, not adding a second time: %s\n", sensor_id);
 					g_free(sensor_id);
 					return FALSE;
 				}
@@ -269,6 +280,7 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 	}
 
 	/* try to load the icon */
+	g_debug("loading icon %s\n", icon_filename);
 	icon = gdk_pixbuf_new_from_file_at_size(icon_filename, DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE, &error);
 	if (error) {
 		 g_warning ("Could not load icon: %s\nTrying with default icon.\n", error->message);
@@ -286,6 +298,7 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 		 }
 	}
 
+	
 	/* then add sensor as a child under interface node - ie assume
 	 * we either found it or created it - the inteface node that
 	 * is */
@@ -309,7 +322,6 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 			   ALARM_ENABLE_COLUMN, alarm_enable,
 			   ALARM_COMMAND_COLUMN, alarm_command,
 			   ALARM_TIMEOUT_COLUMN, alarm_timeout,
-			   ALARM_TIMEOUT_INDEX_COLUMN, -1,
 			   MULTIPLIER_COLUMN, multiplier,
 			   OFFSET_COLUMN, offset,
 			   ICON_FILENAME_COLUMN, icon_filename,
@@ -319,8 +331,34 @@ gboolean sensors_applet_add_sensor_full_details(SensorsApplet *sensors_applet,
 	/* remove reference to icon as tree now has ref */
 	g_object_unref(icon);
 
+	/* create the active sensor */
+	if (enable) {
+		tree_path = gtk_tree_model_get_path(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter);
+		sensors_applet_sensor_enabled(sensors_applet,
+					      tree_path);
+		gtk_tree_path_free(tree_path);
+	}
+
 }	
 
+
+static ActiveSensor *sensors_applet_find_active_sensor(SensorsApplet *sensors_applet,
+							GtkTreePath *path) {
+	GtkTreePath *sensor_tree_path;
+	GList *current_sensor;
+	
+	for (current_sensor = sensors_applet->active_sensors; current_sensor != NULL; current_sensor = g_list_next(current_sensor)) {
+		sensor_tree_path = gtk_tree_row_reference_get_path(((ActiveSensor *)(current_sensor->data))->sensor_row);
+
+		if (gtk_tree_path_compare(path, sensor_tree_path) == 0) {
+			g_debug("found AS...\n");
+			return ((ActiveSensor *)(current_sensor->data));
+		}
+	}
+	return NULL;
+}
+	
+	
 /* path should be the full path to a file representing the sensor (eg
  * /dev/hda or /sys/devices/platform/i2c-0/0-0290/temp1_input) */
 gboolean sensors_applet_add_sensor(SensorsApplet *sensors_applet,
@@ -357,192 +395,7 @@ gboolean sensors_applet_add_sensor(SensorsApplet *sensors_applet,
 	
 	
 /* internal helper functions for updating display etc*/
-/* returns a constant string containing the pango fontsize, defaults
-   to medium if error - this value is owned and cannot be freed by the
-   caller */
-static const gchar *sensors_applet_get_font_size(SensorsApplet *sensors_applet) {
-	FontSize font_size;
 
-	font_size = panel_applet_gconf_get_int(sensors_applet->applet, FONT_SIZE, NULL);
-	switch (font_size) {
-	case XX_LARGE:
-		return XX_LARGE_TEXT;
-	case X_LARGE:
-		return X_LARGE_TEXT;
-	case LARGE:
-		return LARGE_TEXT;
-	case MEDIUM:
-		return MEDIUM_TEXT;
-	case SMALL:
-		return SMALL_TEXT;
-	case X_SMALL:
-		return X_SMALL_TEXT;
-	case XX_SMALL:
-		return XX_SMALL_TEXT;
-	default:
-		return MEDIUM_TEXT;
-	}
-}
-
- /* HELPER FUNCTIONS for managing labels and values lists */ 	 
- /* removes remainder of list links and frees widgets inside list */ 	 
- static void sensors_applet_crop_list(GSList **list, GSList **current_element) { 	 
-         while (*current_element != NULL) { 	 
-                 GSList *old_element = *current_element; 	 
-                 gtk_widget_destroy(GTK_WIDGET(old_element->data)); 	 
-                 *list = g_slist_remove_link(*list, old_element); 	 
-                 *current_element = g_slist_next(*current_element); 	 
-         } 	 
- } 	 
-  	 
- /* take the given text and add if there is already a node at the
-  * given element, change its text to this, or create a new node and
-  * point current_element at the newly created node */ 	 
-static void sensors_applet_add_label_to_list(GSList **list, GSList **current_element, const gchar *text) { 	 
-	/* if have reached end of list, need to create a new node etc */ 	 
-	if (*current_element == NULL) { 	 
-		/* create a new label and add it to 	 
-		   the list and the display */ 	 
-		*list = g_slist_append(*list, g_object_new(GTK_TYPE_LABEL,
-							   "use-markup", TRUE,
-							   "label", text,
-							   NULL)); 	 
-  	 
-		/* if we have just created the first 	 
-		   node in the list, we will want to be 	 
-		   pointing at it now */ 	 
-		if (g_slist_length(*list) == 1) { 	 
-			*current_element = *list; 	 
-		} else { 	 
-			/* if we just appended a new 	 
-			   node, we will want to be 	 
-			   pointing at this as the 	 
-			   current node */ 	 
-			*current_element = g_slist_next(*current_element); 	 
-			
-		} 	 
-	} else {
-		  gtk_label_set_label(GTK_LABEL((*current_element)->data), text);
-	} 	 
-}
-
-static void sensors_applet_add_image_to_list(GSList **list, GSList **current_element, GdkPixbuf *pixbuf) { 	 
-
-	GtkWidget *image;
-         /* if have reached end of list, need to create a new node etc */ 	 
-	if (*current_element == NULL) {
-		/* create a new image and add it to 	 
-		   the list and the display */ 	 
-		 image = gtk_image_new_from_pixbuf(pixbuf);
-                 *list = g_slist_append(*list, image); 	 
-  	 
-                 /* if we have just created the first 	 
-                    node in the list, we will want to be 	 
-                    pointing at it now */ 	 
-                 if (g_slist_length(*list) == 1) { 	 
-                         *current_element = *list; 	 
-                 } else { 	 
-                         /* if we just appended a new 	 
-                            node, we will want to be 	 
-                            pointing at this as the 	 
-                            current node */ 	 
-                         *current_element = g_slist_next(*current_element); 	 
-  	 
-                 } 	 
-         } else { 	 
-		 /* and set to new one if needed */
-		 if (gtk_image_get_pixbuf(GTK_IMAGE((*current_element)->data)) != pixbuf) {
-			 gtk_image_set_from_pixbuf(GTK_IMAGE((*current_element)->data),
-						   pixbuf);
-		 }
-		 
-		 
-         } 	 
-}
-
-static gboolean sensors_applet_execute_alarm(const gchar *alarm_command) {
-	gnome_execute_shell(NULL, alarm_command);
-	return TRUE;
-}
-
-static guint sensors_applet_add_alarm(SensorsApplet *sensors_applet, const gchar *alarm_command, guint alarm_timeout) {
-	gchar *command = g_strdup(alarm_command);
-	sensors_applet->alarm_commands = g_slist_prepend(sensors_applet->alarm_commands, command);
-	return g_timeout_add(alarm_timeout,
-			     (GSourceFunc)sensors_applet_execute_alarm,
-			     command);
-}
-
-static void sensors_applet_alarm_on(SensorsApplet *sensors_applet, GtkTreeIter *iter) {
-	gchar *alarm_command;
-	gint alarm_timeout_index;
-	guint alarm_timeout;
-	gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors),
-			   iter,
-			   ALARM_TIMEOUT_INDEX_COLUMN, &alarm_timeout_index,
-			   -1);
-	
-	if (alarm_timeout_index == -1) {
-		/* alarm is not currently on */
-
-		gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors),
-				   iter,
-				   ALARM_COMMAND_COLUMN, &alarm_command,
-				   ALARM_TIMEOUT_COLUMN, &alarm_timeout,
-				   -1);
-		/* execute alarm once, then add to time to
-		   keep repeating it */
-		gnome_execute_shell(NULL, alarm_command);
-		if (alarm_timeout <= 0) {
-			alarm_timeout_index = sensors_applet_add_alarm(sensors_applet, alarm_command, G_MAXINT);
-		} else {
-			alarm_timeout_index = sensors_applet_add_alarm(sensors_applet, alarm_command, alarm_timeout * 1000);
-		}
-		gtk_tree_store_set(sensors_applet->sensors,
-				   iter,
-				   ALARM_TIMEOUT_INDEX_COLUMN, alarm_timeout_index,
-				   -1);
-		g_free(alarm_command);
-	} 
-}
-
-/* needs to be able to be called by the config dialog when the alarm
- * command changes */
-void sensors_applet_alarm_off(SensorsApplet *sensors_applet, GtkTreeIter *iter) {
-	gint alarm_timeout_index;
-	gchar *alarm_command;
-	GSList *current_alarm_command;
-
-	gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors),
-			   iter,
-			   ALARM_TIMEOUT_INDEX_COLUMN, &alarm_timeout_index,
-			   -1);
-	if (alarm_timeout_index != -1) {
-		/* alarm is on, cycle through list of alarm_commands
-		 * and free this one */
-
-		gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors),
-				   iter,
-				   ALARM_COMMAND_COLUMN, &alarm_command,
-				   -1);
-		for (current_alarm_command = sensors_applet->alarm_commands; current_alarm_command != NULL; current_alarm_command = g_slist_next(current_alarm_command)) {
-			if (g_strrstr(alarm_command, ((gchar *)(current_alarm_command->data))) == alarm_command) {
-				g_free(current_alarm_command->data);
-				sensors_applet->alarm_commands = g_slist_remove_link(sensors_applet->alarm_commands, current_alarm_command);
-				g_slist_free_1(current_alarm_command);
-				break;
-			}
-		}
-		g_free(alarm_command);
-		if (!g_source_remove(alarm_timeout_index)) {
-			g_printf("Error removing alarm source\n");
-		}
-		gtk_tree_store_set(sensors_applet->sensors,
-				   iter,
-				   ALARM_TIMEOUT_INDEX_COLUMN, -1,
-				   -1);
-	}
-}
 
 /* should be called as a g_container_foreach at the start of
  * pack_display if ythe table already exists to remove but keep alive
@@ -570,17 +423,17 @@ static void sensors_applet_pack_display_cleanup_refs_cb(GtkWidget *widget,
 }
 						 
 static void sensors_applet_pack_display(SensorsApplet *sensors_applet) {
+	/* note the if () around each widget is to ensure we only
+	 * operate on those that actually exist */
 	static GtkLabel *no_sensors_enabled_label = NULL;
-	int num_active_sensors = 0, num_sensors_per_group, cols, rows, i, j;
+	gint num_active_sensors = 0, num_sensors_per_group, cols, rows, i, j;
 	GList *old_table_children = NULL;
-	GList *current_child;
 
-	GSList *current_value;
-	GSList *current_label;
-	GSList *current_icon;
+	GList *current_sensor;
+
 	gint display_mode;
 
-	num_active_sensors = g_slist_length(sensors_applet->values);
+	num_active_sensors = g_list_length(sensors_applet->active_sensors);
 	display_mode = panel_applet_gconf_get_int(sensors_applet->applet, DISPLAY_MODE, NULL);
 
 	num_sensors_per_group = panel_applet_gconf_get_int(sensors_applet->applet, NUM_SENSORS, NULL);
@@ -655,63 +508,76 @@ static void sensors_applet_pack_display(SensorsApplet *sensors_applet) {
 
 	}
 
-	current_value = sensors_applet->values;
-	current_label = sensors_applet->labels;
-	current_icon = sensors_applet->icons;
-
 	/* pack icons / labels and values into table */
-	
+	current_sensor = sensors_applet->active_sensors;
+
 	/* if showing labels need to pack these first */
 	if (display_mode != DISPLAY_NONE) {
 		/* loop through columns */
-		for (i = 0; current_label != NULL && current_value != NULL && current_icon != NULL && i < cols; /* increments depends on how we lay them out - see below */) {
-			
+		for (i = 0; current_sensor != NULL && i < cols; /* increments depends on how we lay them out - see below */) {
+		
 			/* loop through rows in a column */
-			for (j = 0; current_label != NULL && current_value != NULL && current_icon != NULL && j < rows; /* see bottom of for loop*/) {
+			for (j = 0; current_sensor && j < rows; /* see bottom of for loop*/) {
 				/* attach label / icon at this point */
 				if (display_mode == DISPLAY_ICON) {
-					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
-								  GTK_WIDGET(current_icon->data),
-							  i, i + 1,
-							  j, j + 1);
+					if (((ActiveSensor *)(current_sensor->data))->icon) {
+						gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
+									  GTK_WIDGET(((ActiveSensor *)(current_sensor->data))->icon),
+									  i, i + 1,
+									  j, j + 1);
+					}
 				} else {
-					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
-								  GTK_WIDGET(current_label->data),
-							  i, i + 1,
-							  j, j + 1);
-				}				
+					if (((ActiveSensor *)(current_sensor->data))->label) {
+						gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
+									  GTK_WIDGET(((ActiveSensor *)(current_sensor->data))->label),
+									  i, i + 1,
+									  j, j + 1);
+					}				
+				}
 				/* now attach sensor value to either
 				   row below or column next to */
 				if (panel_applet_gconf_get_bool(sensors_applet->applet, LABELS_INLINE, NULL)) { 
 					/* left align labels */
-                                        gtk_misc_set_alignment(GTK_MISC(current_icon->data), 0.0, 0.5);
-					gtk_misc_set_alignment(GTK_MISC(current_label->data), 0.0, 0.5); 	 
-					gtk_misc_set_alignment(GTK_MISC(current_value->data), 0.0, 0.5);
+					if (((ActiveSensor *)(current_sensor->data))->icon) {
+						gtk_misc_set_alignment(GTK_MISC(((ActiveSensor *)(current_sensor->data))->icon), 0.0, 0.5);
+					}
+					if (((ActiveSensor *)(current_sensor->data))->label) {
+						gtk_misc_set_alignment(GTK_MISC(((ActiveSensor *)(current_sensor->data))->label), 0.0, 0.5); 	 
+					}
+					if (((ActiveSensor *)(current_sensor->data))->value) {
+						gtk_misc_set_alignment(GTK_MISC(((ActiveSensor *)(current_sensor->data))->value), 0.0, 0.5);
+					}
  
 
-
 					 /* place value next to label */
-					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
-								  GTK_WIDGET(current_value->data),
-								  i + 1, i + 2,
-								  j, j + 1);
-					
+					if (((ActiveSensor *)(current_sensor->data))->value) {
+						gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
+									  GTK_WIDGET(((ActiveSensor *)(current_sensor->data))->value),
+									  i + 1, i + 2,
+									  j, j + 1);
+					}
 					j++;
 				} else { /* place value below label */
 					/* center align labels */ 	 
-					gtk_misc_set_alignment(GTK_MISC(current_icon->data), 0.5, 0.5); 	 
-					gtk_misc_set_alignment(GTK_MISC(current_label->data), 0.5, 0.5); 	 
-					gtk_misc_set_alignment(GTK_MISC(current_value->data), 0.5, 0.5); 	 
+					if (((ActiveSensor *)(current_sensor->data))->icon) {
+						gtk_misc_set_alignment(GTK_MISC(((ActiveSensor *)(current_sensor->data))->icon), 0.5, 0.5);
+					}
+					if (((ActiveSensor *)(current_sensor->data))->label) {
+						gtk_misc_set_alignment(GTK_MISC(((ActiveSensor *)(current_sensor->data))->label), 0.5, 0.5);
+					}
+					if (((ActiveSensor *)(current_sensor->data))->value) {
+						gtk_misc_set_alignment(GTK_MISC(((ActiveSensor *)(current_sensor->data))->value), 0.5, 0.5); 	 
+					}
  
-					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
-								  GTK_WIDGET(current_value->data),
-								  i, i + 1,
-								  j + 1, j + 2);
+					if (((ActiveSensor *)(current_sensor->data))->value) {
+						gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
+									  GTK_WIDGET(((ActiveSensor *)(current_sensor->data))->value),
+									  i, i + 1,
+									  j + 1, j + 2);
+					}
 					j += 2;
 				}
-				current_label = g_slist_next(current_label);
-				current_icon = g_slist_next(current_icon);
-				current_value = g_slist_next(current_value);
+				current_sensor = g_list_next(current_sensor);
 
 			} /* end row loop */
 			/* now increment column index as needed */
@@ -726,14 +592,16 @@ static void sensors_applet_pack_display(SensorsApplet *sensors_applet) {
 
 		
 	} else { /* not showing labels or icons so just pack values */
-		for (i = 0; current_value != NULL && i < cols; ++i) {
-			for (j = 0; current_value!= NULL && j < rows; ++j) {
-				gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
-							  GTK_WIDGET(current_value->data),
-							  i, i + 1,
-							  j, j + 1);
+		for (i = 0; current_sensor != NULL && i < cols; ++i) {
+			for (j = 0; current_sensor!= NULL && j < rows; ++j) {
+				if (((ActiveSensor *)(current_sensor->data))->value) {
+					gtk_table_attach_defaults(GTK_TABLE(sensors_applet->table),
+								  GTK_WIDGET(((ActiveSensor *)(current_sensor->data))->value),
+								  i, i + 1,
+								  j, j + 1);
+				}
 
-				current_value = g_slist_next(current_value);
+				current_sensor = g_list_next(current_sensor);
 			}
 		}
 		
@@ -744,231 +612,133 @@ static void sensors_applet_pack_display(SensorsApplet *sensors_applet) {
 				      old_table_children);
 		g_list_free(old_table_children);
 	}
+	gtk_widget_show_all(GTK_WIDGET(sensors_applet->applet));
 
 } 	    
 
-static void sensors_applet_update_display_only(SensorsApplet *sensors_applet) {
-	sensors_applet_update_display(sensors_applet,
-				      FALSE);
+void sensors_applet_display_layout_changed(SensorsApplet *sensors_applet) {
+	sensors_applet_pack_display(sensors_applet);
 }
 
-void sensors_applet_update_display(SensorsApplet *sensors_applet,
-				   gboolean repack_display) {
-	/* walk through tree structure and for each enabled sensor
-	 * call it's get_sensor_value function and then update labels
-	 * etc */
+void sensors_applet_alarm_off(SensorsApplet *sensors_applet, GtkTreePath *path) {
+	ActiveSensor *active_sensor;
 
-	gchar *tooltip;
-	gint number_of_active_sensors = 0;
-	static gint last_number_of_active_sensors = 0;
-
-	/* to iterate through the sensors data structure */
-	GtkTreeIter interfaces_iter, sensors_iter;
-	gboolean not_end_of_interfaces = TRUE, not_end_of_sensors = TRUE;
-
-	gchar *sensor_path = NULL;
-	gchar *sensor_id = NULL;
-	gchar *sensor_label = NULL;
-	SensorType sensor_type;
-	SensorInterface sensor_interface;
-	gboolean sensor_enabled;
-	gdouble sensor_alarm_value;
-	gboolean sensor_alarm_enabled;
-	AlarmType sensor_alarm_type;
-	gdouble sensor_multiplier;
-	gdouble sensor_offset;
-	gdouble sensor_value;
-
-	/* to build the list of labels as we go */
-	gchar *label_text = NULL;
-	gchar *value_text = NULL;
-	const gchar *font_size;
-	gchar *old_value_text;
-	GdkPixbuf *icon;
-
-	GSList *current_label = NULL;
-	GSList *current_value = NULL;
-	GSList *current_icon = NULL;
-	
-	GError *error = NULL;
-
-	g_assert(sensors_applet->sensors != NULL);
-
-	/* get the first label in the list */
-	current_label = sensors_applet->labels;
-	current_value = sensors_applet->values;
-	current_icon = sensors_applet->icons;
-	
-	/* now step through the GtkTreeStore sensors to
-	   find which sensors are enabled */
-	for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(sensors_applet->sensors), &interfaces_iter); not_end_of_interfaces; not_end_of_interfaces = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_applet->sensors), &interfaces_iter)) {
-		/* reset sensors sentinel */
-		not_end_of_sensors = TRUE;
-		for (gtk_tree_model_iter_children(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter, &interfaces_iter); not_end_of_sensors; not_end_of_sensors = gtk_tree_model_iter_next(GTK_TREE_MODEL(sensors_applet->sensors), &sensors_iter)) {
-			gtk_tree_model_get(GTK_TREE_MODEL(sensors_applet->sensors), 
-					   &sensors_iter,
-					   PATH_COLUMN, &sensor_path,
-					   ID_COLUMN, &sensor_id,
-					   LABEL_COLUMN, &sensor_label,
-					   INTERFACE_COLUMN, &sensor_interface,
-					   SENSOR_TYPE_COLUMN, &sensor_type,
-					   ENABLE_COLUMN, &sensor_enabled,
-					   ALARM_VALUE_COLUMN, &sensor_alarm_value,
-					   ALARM_TYPE_COLUMN, &sensor_alarm_type,
-					   ALARM_ENABLE_COLUMN, &sensor_alarm_enabled,
-					   MULTIPLIER_COLUMN, &sensor_multiplier,
-					   OFFSET_COLUMN, &sensor_offset,
-					   ICON_PIXBUF_COLUMN, &icon,
-					   -1);
-			
-			if (sensor_enabled) {
-				/* get font size */
-				font_size = sensors_applet_get_font_size(sensors_applet);
-				
-				sensor_value = sensors_applet->get_sensor_value[sensor_interface](sensor_path,
-												  sensor_id,
-												  sensor_type,
-												  &error);
-				
-				
-				if (error) {
-					value_text = g_strdup(error->message);
-					g_error_free(error);
-					error = NULL;
-				} else { 
-					switch (sensor_type) {
-					case TEMP_SENSOR: 
-						if (panel_applet_gconf_get_bool(sensors_applet->applet, FARENHEIT, NULL)) {
-							sensor_value = (9.0 * sensor_value / 5.0) + 32.0;
-							sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
-							value_text = g_strdup_printf("%2.0f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_UNITS, NULL) ? "\302\260F" : ""));
-						} else {
-							sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
-							value_text = g_strdup_printf("%2.0f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_UNITS, NULL) ? "\302\260C" : ""));
-						}
-						break;
-						
-					case FAN_SENSOR:
-						sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
-						value_text = g_strdup_printf("%4.0f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_UNITS, NULL) ? _("RPM") : ""));
-						break;
-						
-					case VOLTAGE_SENSOR:
-						sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
-						value_text = g_strdup_printf("%4.2f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_UNITS, NULL) ? _("V") : ""));
-						break;
-						
-					case CURRENT_SENSOR:
-						sensor_value = (sensor_value * sensor_multiplier) + sensor_offset;
-						value_text = g_strdup_printf("%4.2f%s", sensor_value, (panel_applet_gconf_get_bool(sensors_applet->applet, SHOW_UNITS, NULL) ? _("A") : ""));
-						break;
-						
-					default:
-						g_assert_not_reached();	
-						
-					} /* end switch(sensor_type) */
-				        old_value_text = value_text;
-					
-					if (sensor_alarm_enabled) {
-						switch (sensor_alarm_type) {
-						case ALARM_WHEN_VALUE_GREATER_THAN_THRESHOLD:
-							if (sensor_value >= sensor_alarm_value) {
-								/* make value
-								 * text red with
-								 correct font
-								 size and do
-								 alarm command */
-								value_text = g_markup_printf_escaped("<span foreground=\"#FF0000\" size=\"%s\">%s</span>", font_size, old_value_text);
-								sensors_applet_alarm_on(sensors_applet, &sensors_iter);
-							} else {
-								/* just do font size */
-								value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
-								sensors_applet_alarm_off(sensors_applet, &sensors_iter);
-							}				
-							break;
-						case ALARM_WHEN_VALUE_LESS_THAN_THRESHOLD:
-							if (sensor_value <= sensor_alarm_value) {
-								/* make value
-								 * text red with
-								 correct font
-								 size and do
-								 alarm command */
-								value_text = g_markup_printf_escaped("<span foreground=\"#FF0000\" size=\"%s\">%s</span>", font_size, old_value_text);
-								sensors_applet_alarm_on(sensors_applet, &sensors_iter);
-						} else {
-								/* just do font size */
-								value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
-								sensors_applet_alarm_off(sensors_applet, &sensors_iter);
-							}				
-							break;
-						default: 
-							g_assert_not_reached();
-						} /* end
-						   * switch(sensor_alarm_type) */
-
-					} else {
-						/* just do font size */
-						value_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, old_value_text);
-						sensors_applet_alarm_off(sensors_applet, &sensors_iter);
-					}
-					
-					g_free(old_value_text);
-				} /* end of else from if (error) */
-				sensors_applet_add_label_to_list(&(sensors_applet->values), &current_value, value_text);
-				g_free(value_text);
-				
-				/* advance to next element in list */
-				current_value = g_slist_next(current_value);
-				
-				/* create label text with font size
-				 * formatting */
-				label_text = g_markup_printf_escaped("<span size=\"%s\">%s</span>", font_size, sensor_label);
-				sensors_applet_add_label_to_list(&(sensors_applet->labels), &current_label, label_text);
-				g_free(label_text);
-				current_label = g_slist_next(current_label);
-
-				/* add icon to icons list */
-				sensors_applet_add_image_to_list(&(sensors_applet->icons), &current_icon, icon);
-				/* g_object_unref(icon) */
-				current_icon = g_slist_next(current_icon);
-				
-			} /* end if (sensor_enabled) */
-			g_free(sensor_path);
-			g_free(sensor_id);
-			g_free(sensor_label);
-			g_object_unref(icon);
-
-		} /* end for loop of sensors */
-	} /* end for loop of interfaces */
-	
-	/* now remove any remaining elements in either list to free up
-	   memory */
-	sensors_applet_crop_list(&(sensors_applet->values), &current_value);
-	sensors_applet_crop_list(&(sensors_applet->labels), &current_label);
-	sensors_applet_crop_list(&(sensors_applet->icons), &current_icon);
-	
-	number_of_active_sensors = g_slist_length(sensors_applet->values);
-
-	/* now repack labels into table update display of applets */
-	if (repack_display || last_number_of_active_sensors != number_of_active_sensors) {
-		sensors_applet_pack_display(sensors_applet);
+	if ((active_sensor = sensors_applet_find_active_sensor(sensors_applet,
+							       path)) != NULL) {
+		active_sensor_alarm_off(active_sensor,
+					path);
 	}
-	last_number_of_active_sensors = number_of_active_sensors;
+}
+				
+void sensors_applet_sensor_enabled(SensorsApplet *sensors_applet,
+				   GtkTreePath *path) {
+	ActiveSensor *active_sensor;
 
+	g_debug("creating new active sensor\n");
+	g_assert(sensors_applet != NULL);
+	g_assert(path != NULL);
+
+	active_sensor = g_new0(ActiveSensor, 1);
+	active_sensor->sensor_row = gtk_tree_row_reference_new(GTK_TREE_MODEL(sensors_applet->sensors),
+							       path);
+	active_sensor->alarm_timeout_id = -1;
+	
+	/* keep list sorted */
+	sensors_applet->active_sensors = g_list_insert_sorted(sensors_applet->active_sensors, active_sensor, (GCompareFunc)active_sensor_compare);
+	
+	active_sensor_update(active_sensor, sensors_applet);
+
+	sensors_applet_pack_display(sensors_applet);
+}
+
+void sensors_applet_sensor_disabled(SensorsApplet *sensors_applet,
+				    GtkTreePath *path) {
+
+	GtkTreeModel *model;
+	ActiveSensor *active_sensor;
+
+	g_assert(sensors_applet != NULL);
+	g_assert(path != NULL);
+
+	if ((active_sensor = sensors_applet_find_active_sensor(sensors_applet,
+							       path)) != NULL) {
+		g_debug("Destroying active sensor...\n");
+		
+		g_debug("-- removing from list...\n");
+		sensors_applet->active_sensors = g_list_remove(sensors_applet->active_sensors,
+			      active_sensor);
+		g_debug("-- repacking display....\n");
+		sensors_applet_pack_display(sensors_applet);
+
+		/* destroy the current_sensor */
+		g_debug("-- destroying active sensor label...\n");
+		gtk_object_destroy(GTK_OBJECT(active_sensor->label));
+		g_debug("-- destroying active sensor icon...\n");
+		gtk_object_destroy(GTK_OBJECT(active_sensor->icon));
+		g_debug("-- destroying active sensor value...\n");
+		gtk_object_destroy(GTK_OBJECT(active_sensor->value));
+		if (active_sensor->alarm_timeout_id >= 0) {
+			active_sensor_alarm_off(active_sensor);
+		}
+		g_free(active_sensor);
+	}
+}
+
+
+void sensors_applet_update_sensor(SensorsApplet *sensors_applet,
+				  GtkTreePath *path) {
+	ActiveSensor *active_sensor;
+
+	g_assert(sensors_applet != NULL);
+	g_assert(path != NULL);
+
+	if ((active_sensor = sensors_applet_find_active_sensor(sensors_applet,
+							       path)) != NULL) {
+		active_sensor_update(active_sensor, 
+				     sensors_applet);
+	}
+}
+ 
+void sensors_applet_icon_changed(SensorsApplet *sensors_applet,
+				 GtkTreePath *path) {
+	ActiveSensor *active_sensor;
+	
+	g_assert(sensors_applet != NULL);
+	g_assert(path != NULL);
+	
+	if ((active_sensor = sensors_applet_find_active_sensor(sensors_applet,
+							       path)) != NULL) {
+		active_sensor_icon_changed(active_sensor,
+					   sensors_applet);
+	}
+}
+/**
+ * Cycle thru ActiveSensors and update them all
+ */
+void sensors_applet_update_active_sensors(SensorsApplet *sensors_applet) {
+	gchar *tooltip;
+	unsigned long int num_active_sensors;
+     
+	g_assert(sensors_applet != NULL);
+	g_assert(sensors_applet->active_sensors != NULL);
+	
+	num_active_sensors = (unsigned long int)g_list_length(sensors_applet->active_sensors);
+
+	g_list_foreach(sensors_applet->active_sensors,
+		       (GFunc)active_sensor_update,
+		       sensors_applet);
 
 	tooltip = g_strdup_printf("%s\n%d %s",
 				  _("Sensors Applet"),
-				  number_of_active_sensors,
-				  ngettext("sensor enabled", 
-					   "sensors enabled",
-					   number_of_active_sensors));
+				  num_active_sensors,
+				  ngettext(_("sensor enabled"), 
+					   _("sensors enabled"),
+					   num_active_sensors));
 	gtk_tooltips_set_tip(sensors_applet->tooltips, GTK_WIDGET(sensors_applet->applet), tooltip, "");
 	g_free(tooltip);
 
-	gtk_widget_show_all(GTK_WIDGET(sensors_applet->applet));
-
 	return;
-
+	
 }
 
 static void sensors_applet_write_defaults_to_gconf(SensorsApplet *sensors_applet) {
@@ -986,12 +756,17 @@ static void sensors_applet_write_defaults_to_gconf(SensorsApplet *sensors_applet
 static void sensors_applet_setup_sensors_interfaces(SensorsApplet *sensors_applet) {
 	acpi_sensors_interface_init(sensors_applet);
 	hddtemp_sensors_interface_init(sensors_applet);
+#ifdef HAVE_LIBSENSORS
+	libsensors_sensors_interface_init(sensors_applet);
+#else
 	i2c_proc_sensors_interface_init(sensors_applet);
 	i2c_sys_sensors_interface_init(sensors_applet);
+#endif
 	i8k_sensors_interface_init(sensors_applet);
 	ibm_acpi_sensors_interface_init(sensors_applet);
 	omnibook_sensors_interface_init(sensors_applet);
 	pmu_sys_sensors_interface_init(sensors_applet);
+	smu_sys_sensors_interface_init(sensors_applet);
 }
 
 
@@ -1024,6 +799,7 @@ void sensors_applet_init(SensorsApplet *sensors_applet) {
 	} else {
 		/* setup sensors from stored gconf values */
 		sensors_applet_gconf_setup_sensors(sensors_applet);
+		g_debug("done setting up from gconf\n");
 	}
 	/* now do any setup needed manually */
 	sensors_applet_setup_sensors_interfaces(sensors_applet);
@@ -1042,9 +818,10 @@ void sensors_applet_init(SensorsApplet *sensors_applet) {
 
 	sensors_applet->tooltips = gtk_tooltips_new();
 	gtk_tooltips_set_tip(sensors_applet->tooltips, GTK_WIDGET(sensors_applet->applet), _("Sensors Applet"), "");
-	sensors_applet_update_display(sensors_applet, TRUE);
+	sensors_applet_update_active_sensors(sensors_applet);
+	sensors_applet_pack_display(sensors_applet);
 
-	sensors_applet->timeout_id = g_timeout_add(panel_applet_gconf_get_int(sensors_applet->applet, TIMEOUT, NULL), (GSourceFunc)sensors_applet_update_display_only, sensors_applet);
+	sensors_applet->timeout_id = g_timeout_add(panel_applet_gconf_get_int(sensors_applet->applet, TIMEOUT, NULL), (GSourceFunc)sensors_applet_update_active_sensors, sensors_applet);
 
 	gtk_widget_show_all(GTK_WIDGET(sensors_applet->applet));
 }
