@@ -22,16 +22,19 @@
 #include <panel-applet.h>
 
 typedef struct _SensorsApplet SensorsApplet;
+typedef struct _ActiveSensor ActiveSensor;
 
 #include "prefs-dialog.h"
 
 #define SENSORS_APPLET_ICON PIXMAPS_DIR "sensors-applet-icon.png"
-#define CPU_TEMP_ICON PIXMAPS_DIR "cpu-temp-icon.png"
-#define HDD_TEMP_ICON PIXMAPS_DIR "hdd-temp-icon.png"
-#define BATTERY_TEMP_ICON PIXMAPS_DIR "battery-temp-icon.png"
-#define MEM_TEMP_ICON PIXMAPS_DIR "mem-temp-icon.png"
-#define GPU_TEMP_ICON PIXMAPS_DIR "gpu-temp-icon.png"
-#define GENERIC_TEMP_ICON PIXMAPS_DIR "generic-temp-icon.png"
+
+/* device icons */
+#define CPU_ICON PIXMAPS_DIR "cpu-icon.png"
+#define HDD_ICON PIXMAPS_DIR "hdd-icon.png"
+#define BATTERY_ICON PIXMAPS_DIR "battery-icon.png"
+#define MEM_ICON PIXMAPS_DIR "mem-icon.png"
+#define GPU_ICON PIXMAPS_DIR "gpu-icon.png"
+#define GENERIC_ICON PIXMAPS_DIR "generic-icon.png"
 #define FAN_ICON PIXMAPS_DIR "fan-icon.png"
 #define VOLTAGE_ICON NULL
 
@@ -45,8 +48,10 @@ typedef enum {
 	I2C_PROC,
 	IBM_ACPI,
 	I8K,
+	LIBSENSORS,
 	OMNIBOOK,
 	PMU_SYS,
+	SMU_SYS,
 	N_SENSOR_INTERFACES
 } SensorInterface;
 
@@ -58,8 +63,10 @@ static const gchar *sensor_interface[] = {
 	"i2c-proc",
 	"ibm-acpi",
 	"i8k",
+	"libsensors",
 	"omnibook",
 	"pmu-sys",
+	"smu-sys",
 };
 
 /* enumeration used to identify columns in the GtkTreeStore data
@@ -78,7 +85,6 @@ enum {
 	ALARM_ENABLE_COLUMN,
 	ALARM_COMMAND_COLUMN,
 	ALARM_TIMEOUT_COLUMN,
-	ALARM_TIMEOUT_INDEX_COLUMN,
 	MULTIPLIER_COLUMN,
 	OFFSET_COLUMN,
 	ICON_FILENAME_COLUMN,
@@ -119,6 +125,16 @@ typedef enum {
 	XX_SMALL
 } FontSize;
 
+/* constants for scaling icons as font size changes */
+typedef enum {
+	XX_SMALL_ICON_SIZE = 12,
+	X_SMALL_ICON_SIZE = 16,
+	SMALL_ICON_SIZE = 20,
+	MEDIUM_ICON_SIZE = 24,
+	LARGE_ICON_SIZE = 28,
+	X_LARGE_ICON_SIZE = 32,
+	XX_LARGE_ICON_SIZE = 36
+} IconSize;
 
 /* should always return the value in degrees celcius, revolutions per
  * minute, volts, amps, etc */
@@ -146,9 +162,7 @@ struct _SensorsApplet {
 	/* primary table to contain the panel dispay - we pack the
 	 * list of labels and sensor values into this container */
 	GtkWidget *table;
-	GSList *labels;
-	GSList *icons;
-	GSList *values;
+	GList *active_sensors;
 	GtkTooltips *tooltips;
 
 	/* list of gchar* to the alarm commands, which have been
@@ -169,8 +183,11 @@ struct _SensorsApplet {
 
 /* non-static function prototypes */
 void sensors_applet_init(SensorsApplet *sensors_applet);
-void sensors_applet_update_display(SensorsApplet *sensors_applet,
-				   gboolean repack_display);
+void sensors_applet_sensor_enabled(SensorsApplet *sensors_applet,
+				   GtkTreePath *path);
+void sensors_applet_sensor_disabled(SensorsApplet *sensors_applet,
+				    GtkTreePath *path);
+void sensors_applet_update_active_sensors(SensorsApplet *sensors_applet);
 void sensors_applet_register_sensors_interface(SensorsApplet *sensors_applet,
 					       SensorInterface interface,
 					       GetSensorValueFunction get_sensor_value_function);
@@ -199,5 +216,14 @@ gboolean sensors_applet_add_sensor(SensorsApplet *sensors_applet,
 				   gboolean enable,
 				   SensorType type,
 				   const gchar *icon_filename); 
-void sensors_applet_alarm_off(SensorsApplet *sensors_applet, GtkTreeIter *iter);
+
+/**
+ * to be called by things like prefs dialog to turn off a sensor alarm
+ */
+void sensors_applet_alarm_off(SensorsApplet *sensors_applet, GtkTreePath *path);
+void sensors_applet_update_sensor(SensorsApplet *sensors_applet,
+				  GtkTreePath *path);
+void sensors_applet_display_layout_changed(SensorsApplet *sensors_applet);
+void sensors_applet_icon_changed(SensorsApplet *sensors_applet,
+				 GtkTreePath *path);
 #endif /* SENSORS_APPLET_H */
